/*
 * Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sys/mman.h>
#include <errno.h>
#include <unistd.h>
#include <pthread.h>
#include <semaphore.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include "mpm_transport.h"

#define CONVERT_ULL_TO_ALIAS(x) (0x80000000 + (x - 0x800000000))

#define TEST_DESTINATION1 0x822000000
#define TEST_DESTINATION2 0xa0000000
#define TEST_DESTINATION3 0xa1000000
#define TEST_SOURCE1 0x839000000
#define TEST_SOURCE2 0x830000000
#define TEST_SOURCE3 0x838000000
#define TEST_BUFFER_SIZE 0x400000

#define TEST_GET_DESTINATION (0x820400000)

#define MAX_LINE_LENGTH 64

int check_device_exist(char *st) 
{
    int ret, fd;
    char tmp[6] = "/dev/";
    char dev[MAX_LINE_LENGTH];
    snprintf(dev, strlen(tmp) + strlen(st) + 1, "%s%s", tmp, st);
    fd = open(dev, O_SYNC);
    if (fd < 0) {
    	/* errno 2 is No such file or directory. */
    	if (errno == 2) ret = 0;
    	else ret = errno;
    }
    else {
        close(fd);
        ret = 1;
    }
    return ret;
}

int dev_mem_fd;
void * my_mmap(uint32_t addr, uint32_t size)
{
	void *virt_addr;
    uint32_t page_size, mapaddr, offset;
    page_size = getpagesize();
    mapaddr = addr & (~(page_size - 1));
    offset = addr - mapaddr;
	virt_addr = mmap(0, size + offset, (PROT_READ|PROT_WRITE), MAP_SHARED, dev_mem_fd, (off_t)mapaddr);
	return virt_addr + offset;
}

int verify_data(uint32_t dst, uint32_t length, uint32_t val)
{
	uint32_t *dstBuff;
	uint32_t i;
	printf("->Verifying data for test destination 0x%08x... \t",dst);
	dstBuff = (uint32_t *) my_mmap(dst, length);
	for (i = 0; i<length/4; i++) {
		if ( *(dstBuff+i) != val) {
			printf(" FAILED!\n");
			printf("!! Data verification failed for address 0x%08x\n",dst+i);
			return -1;
		}
	}
	printf(" PASSED!\n");
	return 0;
}

int main()
{
	int i, ret =0;
	int check1, check2;
	uint32_t *srcBuff;
	mpm_transport_h h;
	mpm_transport_h h2;
	mpm_transport_trans_h th;
	mpm_transport_trans_h th2;

	mpm_transport_open_t ocfg = {
		.open_mode	= (O_SYNC|O_RDWR),
		.msec_timeout = 0,
		.serdes_init = 1,
	};

	// Fill in some dummy data...
	dev_mem_fd = open("/dev/mem", (O_RDWR|O_SYNC));
	srcBuff = (uint32_t *) my_mmap(CONVERT_ULL_TO_ALIAS(TEST_SOURCE1), TEST_BUFFER_SIZE);
	for (i = 0; i<TEST_BUFFER_SIZE/4; i++) {
		*(srcBuff+i) = 0xFEEDFEED;
	}
	srcBuff = (uint32_t *) my_mmap(CONVERT_ULL_TO_ALIAS(TEST_SOURCE2), TEST_BUFFER_SIZE);
	for (i = 0; i<TEST_BUFFER_SIZE/4; i++) {
		*(srcBuff+i) = 0xBABEBABE;
	}
	srcBuff = (uint32_t *) my_mmap(CONVERT_ULL_TO_ALIAS(TEST_SOURCE3), TEST_BUFFER_SIZE);
	for (i = 0; i<TEST_BUFFER_SIZE/4; i++) {
		*(srcBuff+i) = 0xDEADDEAD;
	}

	if (check1=check_device_exist("hyperlink0")) {
		printf("testing hyperlink put_initiate 36-bit addr to remote 36-bit addr\n");
		h = mpm_transport_open("arm-loopback-hyplnk64-0",&ocfg);
		if (!h) {
	        printf("h open failed\n");
	        ret = -1;
	    }
	    printf("Writing to 0x%09llx, from 0x%09llx...\n", TEST_DESTINATION1, TEST_SOURCE1);
		th = mpm_transport_put_initiate64(h, TEST_DESTINATION1, TEST_SOURCE1, TEST_BUFFER_SIZE, false, NULL);
		while(mpm_transport_transfer_check(h, th) != 1);
		printf("Verifying data...\n");
		ret = verify_data(CONVERT_ULL_TO_ALIAS(TEST_DESTINATION1), TEST_BUFFER_SIZE, 0xFEEDFEED);
		if (ret == -1) goto end;
		mpm_transport_close(h);
	
		printf("testing hyperlink put_initiate 36-bit addr to remote 32-bit addr\n");
		h = mpm_transport_open("arm-loopback-hyplnk64-0",&ocfg);
		if (!h) {
	        printf("h open failed\n");
	        ret = -1;
	        goto end;
	    }
		mpm_transport_put_initiate64(h, TEST_DESTINATION2, TEST_SOURCE2, TEST_BUFFER_SIZE, true, NULL);
		printf("Verifying data...\n");
		ret = verify_data(TEST_DESTINATION2, TEST_BUFFER_SIZE, 0xBABEBABE);
		if (ret == -1) goto end;
	
		printf("testing hyperlink get_initiate 36-bit addr to remote 36-bit addr\n");
		th2 = mpm_transport_get_initiate64(h, TEST_SOURCE1, TEST_GET_DESTINATION, TEST_BUFFER_SIZE, false, NULL);
		while(mpm_transport_transfer_check(h, th2) != 1);
		ret = verify_data(CONVERT_ULL_TO_ALIAS(TEST_GET_DESTINATION), TEST_BUFFER_SIZE, 0xFEEDFEED);
		if (ret == -1) goto end;
		
		if (check2=check_device_exist("hyperlink1")) {
		    h2 = mpm_transport_open("arm-loopback-hyplnk64-1", &ocfg);
		    if (!h2) {
		        printf("h2 open failed\n");
		        ret = -1;
		        goto end;
		    }
			mpm_transport_put_initiate64(h2, TEST_DESTINATION3, TEST_SOURCE3, TEST_BUFFER_SIZE, true, NULL);
			ret = verify_data(TEST_DESTINATION3, TEST_BUFFER_SIZE, 0xDEADDEAD);
			mpm_transport_close(h2);
		}
		else {
			printf("No hyperlink1 device!\n");
			printf("Skipping hyperlink1 put_initiate64() test...\n");
		}
	}
	else {
		printf("No hyperlink0 device!\n");
		printf("Skipping hyperlink0 put_initiate64() test...\n");
	}

end:
	mpm_transport_close(h);
	if (ret == -1)
		printf("mpm transport64: hyperlink loopback test failed!!\n");
	else
		printf("mpm transport64: hyperlink loopback test finished!\n");
	return 0;
}
